%option no_sysinit
%import textio
%import conv
%import diskio
%import strings
%import sprites
%import floats
%encoding iso
%zeropage basicsafe
%zpreserved $32, $43
%zpreserved $65, $76

VideoHeader {
  &ubyte[4]        ID = $500
  &ubyte        VTYPE = $504
  &ubyte          BPP = $505
  &ubyte          FPS = $506
  &ubyte  SpriteWidth = $507
  &ubyte SpriteHeight = $508
  &uword   FrameWidth = $509
  &uword  FrameHeight = $50B
  &uword  NumFramesLO = $50D
  &uword  NumFramesHI = $50F
  &ubyte        SHint = $511
  &ubyte     VERARate = $512
  &uword AudioFrameSize = $513
  &ubyte[11]    Extra = $515
 }


main  {

&ubyte Volume = $520
&ubyte SoundLFN = $74
&ubyte VideoLFN = $75
&bool InVideo = $530
&bool DoBookMark = $531
&ubyte AUDIO_CTRL = $9F3B

const uword joystick_get = $FF56

defpal:
%asmbinary "DPAL.BIN"

playengine:
%asmbinary "PLAYVIDEO.VIDENGINE.B1.BIN"
endengine:

extsub $A000 = PlayVideo(uword PCMParms @ R0) clobbers(A,X,Y)

; Clears TextBuffer at 1:$B000, GraphBuffer at 0:$0000 and the Sprite Data Buffer at 1:$3000
; (fills with 0)
extsub $A003 = ClearAllBuffers() clobbers(A,X,Y)

extsub $A006 = Engine_SetAudioCode(uword AudParms1 @R0, uword AudParms2 @R1, ubyte AbuffRem @R2 ) clobbers(A)

; Only valid value to pass at the moment is 4 which aligns with the Video LFN for
; Video type 6 & 8 (with embedded Audio).
; eventually both LFN's will be assignable.   For NOW video LFN is 4
; Audio LFN if a seperate file is 6 (same as Vid LFN if embedded)
extsub $A01E = SetAudioLFN(ubyte AudLFN @R0) clobbers(A)

; Should pass either a 0 or a 1 prior to video play.
; 0 for Seperate Audio Track
; 1 for Embedded Audio.
; This was a bit of a cludge to modify the end play
; condition for the video.   For seperate Audio Tracks it waits for two EOF conditions.
; if this variable is 1 either EOF condition will end play.
; i check EOF two places in the code and for some reason can't predict which will return EOF first.
; If playing from a single LFN the 2nd call will not return accurately because ST has already been cleared.
; This is planned for elimination when I can reliably set it internally via SetAudioLFN.
extsub $A021 = SetType(byte Tbyte @R0) clobbers(A)

extsub $A009 = SetSpriteMovie8bit(uword BuffSizes @R3, ubyte SpriteCount @R4, uword EndSprAttrs @R5, uword EndSprData @R6 ) clobbers(A,X)
extsub $A00C = SetSpriteMovie4bit(uword BuffSizes @R3, ubyte SpriteCount @R4, uword EndSprAttrs @R5, uword EndSprData @R6 ) clobbers(A,X)
extsub $A00F = SetBitMapMovie(uword BuffSizes @R3, ubyte WidthByte @R2, ubyte PaletteSize @R4) clobbers(A,X)

; Clears the Graphics Buffer at 0:0000 for 76800 bytes.  (execution time app. 2.7 jiffies)
extsub $A015 = ClearGraphBuffer() clobbers(A,X,Y)

; basically CLS (but always to color 0).  Not sure there's an advantage here but it is also
; called by the Above ClearAllBuffers.
extsub $A018 = ClearTextBuffer() clobbers(A,X,Y)

; Clears the Sprite Buffer at 1:$3000
extsub $A01B = ClearSpriteBuffer() clobbers(A,X,Y)

&ubyte ManualExit = $BFFF

alias f_size_low = cx16.r2
alias f_size_high = cx16.r3


ubyte[85] butterfly = [$8F,$93,$9C,$12,$DF,$92,$20,$20,$20,$20,$20,$12,$A9,$0D,$9A,$12,$A5,$DF,$92,$20,$20,$20,$12,$A9,$A7,
                       $92,$0D,$9F,$12,$B5,$20,$DF,$92,$20,$12,$A9,$20,$B6,$0D,$1E,$20,$B7,$12,$BB,$92,$20,$12,$AC,$92,$B7,
                       $0D,$9E,$20,$AF,$12,$BE,$92,$20,$12,$BC,$92,$AF,$0D,$81,$AA,$12,$20,$92,$A9,$20,$DF,$12,$20,$92,$B4,
                       $0D,$1C,$B6,$A9,$20,$20,$20,$DF,$B5,0]

; This will remain 0 for the time being.
; I am only using 8 bit PCM mono and I probably will eventually
; remove this parameter.   When I first started this project
; I thought I might want the capability of other PCM types so at the moment
; this is a required parameter to Engine_SetAudioCode
;  MY ENGINE IS THE MOST COMPLEX 6502 asm Code I have ever written.  Any
;  changes get made very slowly and methodically. I don't want to break something
;  and not Know HOW I BROKE IT.
ubyte VERAcfgvar = 0

ubyte drivenumber = 8

str FILEID = iso:"SPRV"
str BASENAME = "***REPLACEME***"+"?"*94
uword PAD = 0
str VIDFILE = "MOVIE"+"?"*96
uword PAD2 = 0
str AUDFILE = "?"*96
str VEXT = iso:".SPV"
str AEXT = iso:".SVA"

ubyte i=0


uword FrameBufferSize
uword PalBufferSize
uword NumSprites
uword SpriteAttr_EndAddress
uword SpriteBlock2_DataAddress
ubyte VBuff_255
ubyte VBuff_Remainder
ubyte ABuff_255
ubyte ABuff_Remainder

bool success = false
bool UseBookMark = false

uword SpriteOrigAddress
uword SpriteBuffSize

word XOrg = 0
word YOrg = 0

alias NumFramesHI = cx16.r1
alias NumFramesLO = cx16.r2

float Frame = 0.0
float NumFrames = 0.0
float AudFileSize = 0.0
float vidFilePosition

sub scratch(uword filenameptr) {
        ; -- delete a file on the drive
        &uword command = $600
        poke($600,'S')
        poke($601,':')
        ubyte flen = strings.copy(filenameptr, &command + 2)
        cbm.SETNAM(flen+2, &command)
        cbm.SETLFS(15, drivenumber, 15)
        void cbm.OPEN()
        %asm {{
               bra p8s_Flush15
             }}
     }

asmsub Flush15() clobbers(A,X,Y) {
  %asm {{
          ldx #15
          jsr cbm.CHKIN
          jmp p8s_FExists.flushchars
       }}
    }

asmsub Beep() clobbers(A) {
   %asm {{
       lda #7
       jmp cbm.CHROUT
       }}
   }

asmsub FExists(uword FName @ XY, ubyte NameLength @ A) clobbers(A,X,Y) -> bool @Pc  {
   %asm  {{
             jsr cbm.SETNAM
             lda #11
             ldx p8v_drivenumber
             ldy #0
             jsr cbm.SETLFS
             jsr cbm.OPEN
             lda #11
             jsr cbm.CLOSE
             lda #0
             ldx #<dummy
             ldy #>dummy
             jsr cbm.SETNAM
             lda #15
             ldx p8v_drivenumber
             ldy #15
             jsr cbm.SETLFS
             jsr cbm.OPEN
             ldx #15
             jsr cbm.CHKIN
             jsr cbm.GETIN
             cmp #"0"
             bne NotFound
             jsr cbm.GETIN
             cmp #"0"
             beq Found
NotFound:    jsr flushchars
             clc
             rts
Found:       jsr flushchars
             sec
             rts
flushchars:  jsr cbm.GETIN
             cmp #13
             bne flushchars
             lda #15
             jsr cbm.CLOSE
             jmp cbm.CLRCHN
dummy        .byte 0
; !notreached!
          }}
 }


sub ProgramBanner() {
    i=0
    repeat 85 { cbm.CHROUT(butterfly[i]) i +=1 }
    txt.row(1)
    txt.column(10)
    txt.print ("\x9C*\x9A*\x99* \x05X16 MOVIE PLAYER BETA 7 \x9E*\x81*\x1C*\x05")
    txt.row(3)
    txt.column(10)
    txt.print ("\x9BAUTHOR\x1E: \x05ANTHONY W. HENRY")
    txt.row(5)
    txt.column(10)
    txt.print("\x9ETHIS SOFTWARE IS UNDER THE MIT LICENSE")
    txt.row(6)
    txt.column(10)
    txt.print("    \x05(\x9BPERMISSIVE\x05)")
}

sub ISOUpper(str s) {
  i=0
  while s[i] != 0 {
      if s[i] > 96 and s[i]<123 { s[i] -= 32 }
      i+=1
    }
  }

sub SetDefaultPalette() {
    cx16.VERA_ADDR_L = 0
    cx16.VERA_ADDR_M = $FA
    cx16.VERA_ADDR_H = %00010001
    cx16.memory_copy(defpal,&cx16.VERA_DATA0,512)
    }

sub pal2black() {
    cx16.VERA_ADDR_L = 0
    cx16.VERA_ADDR_M = $FA
    cx16.VERA_ADDR_H = %00010001
    repeat 512 {
       cx16.VERA_DATA0 = 0
      }
  }

sub hideGraphicsLayer() {
    cx16.VERA_DC_VIDEO = cx16.VERA_DC_VIDEO & 239
  }

sub showGraphicsLayer() {
    cx16.VERA_DC_VIDEO = cx16.VERA_DC_VIDEO | 16
  }

sub hideTextLayer() {
    cx16.VERA_DC_VIDEO = cx16.VERA_DC_VIDEO & 223
  }

sub showTextLayer() {
    cx16.VERA_DC_VIDEO = cx16.VERA_DC_VIDEO | 32
  }

sub setMovieVideoMode() {
    pal2black()
    if VideoHeader.VTYPE==1 or VideoHeader.VTYPE==6 { return }
    if VideoHeader.FrameWidth==320 {
      cx16.set_screen_mode($80) }
    else {
      cx16.set_screen_mode(0)
    }
 }

sub setup_SpriteGrid(word X @R11, word Y @R12) {
    alias X1 = cx16.r5
    alias Y1 = cx16.r6
    ubyte sprPal1 = 0
    ubyte sprPal2 = 0
    ubyte sprDepth = 128
    ubyte sprWidByte = $FF
    ubyte sprHeiByte = $FF
    ubyte sprNum = 1
    ubyte sprBank = 0
    X1  = X as uword
    Y1  = Y as uword
    word XLimit = (X + VideoHeader.FrameWidth) as word
    uword sprBlock1Addr = SpriteOrigAddress
    uword sprBlock2Addr = SpriteBlock2_DataAddress
    uword BuffAdd = (VideoHeader.SpriteWidth as uword * VideoHeader.SpriteHeight as uword)

    if VideoHeader.BPP==4 {
       sprDepth = 0
       sprPal1 = 14
       sprPal2 = 15
       sprBank = 1
       BuffAdd = BuffAdd/2
     }

   if VideoHeader.SpriteWidth==64 { sprWidByte=sprites.SIZE_64 }
   if VideoHeader.SpriteWidth==32 { sprWidByte=sprites.SIZE_32 }
   if VideoHeader.SpriteWidth==16 { sprWidByte=sprites.SIZE_16 }
   if VideoHeader.SpriteWidth==8 { sprWidByte= sprites.SIZE_8 }

   if VideoHeader.SpriteHeight==64 { sprHeiByte=sprites.SIZE_64 }
   if VideoHeader.SpriteHeight==32 { sprHeiByte=sprites.SIZE_32 }
   if VideoHeader.SpriteHeight==16 { sprHeiByte=sprites.SIZE_16 }
   if VideoHeader.SpriteHeight==8 { sprHeiByte=sprites.SIZE_8 }

   repeat NumSprites {
         sprites.init(sprNum, sprBank, sprBlock1Addr, sprWidByte, sprHeiByte, sprDepth, sprPal1)
         sprites.pos(sprNum, X1 as word,Y1 as word)
         sprites.init((sprNum+NumSprites) as ubyte, sprBank, sprBlock2Addr, sprWidByte, sprHeiByte, sprDepth, sprPal2)
         sprites.pos((sprNum+NumSprites) as ubyte, X1 as word,Y1 as word)
         sprNum = sprNum+1
         X1 = X1 + VideoHeader.SpriteWidth
         if X1>=XLimit { X1=X as uword
                         Y1=(Y1+VideoHeader.SpriteHeight) as word }
         sprBlock1Addr = sprBlock1Addr + BuffAdd
         sprBlock2Addr = sprBlock2Addr + BuffAdd
     }
  }

sub fopen(ubyte lfn, ubyte device, ubyte sa, str filename) {
    cbm.SETLFS(lfn, device, sa)
    cbm.SETNAM(strings.length(filename), filename)
    void cbm.OPEN()
  }

sub CalcFrameSize() {
    ubyte adjust
    if VideoHeader.BPP==1 { adjust=8
                PalBufferSize = 4}
    if VideoHeader.BPP==2 { adjust=4
                PalBufferSize = 8}
    if VideoHeader.BPP==4 { adjust=2
                PalBufferSize = 32}
    if VideoHeader.BPP==8 { adjust=1
                PalBufferSize = 512}
    FrameBufferSize = ((VideoHeader.FrameWidth as float * VideoHeader.FrameHeight as float) / adjust as float) as uword
    VBuff_255 = (FrameBufferSize / 255 ) as ubyte
    VBuff_Remainder = (FrameBufferSize - (VBuff_255 as uword * 255)) as ubyte
  }

sub CalculateSprites()  {
    NumSprites = (VideoHeader.FrameWidth/VideoHeader.SpriteWidth) * (VideoHeader.FrameHeight/VideoHeader.SpriteHeight)
    SpriteOrigAddress = 0
    SpriteBuffSize = VideoHeader.SpriteHeight as uword * VideoHeader.SpriteWidth as uword
    if VideoHeader.BPP==4 { SpriteBuffSize = SpriteBuffSize/2
                SpriteOrigAddress = $3000}
    SpriteAttr_EndAddress = $FC06 + (NumSprites*16)
    SpriteBlock2_DataAddress = SpriteOrigAddress + FrameBufferSize
    if VideoHeader.BPP==8 { SpriteBlock2_DataAddress = SpriteBlock2_DataAddress + 512 }
    }


sub calc_asize(float filesize, float numframes) -> uword {
        return floats.round(filesize / numframes) as uword
    }

sub CalculateVERARate() -> ubyte {
        float drate =  VideoHeader.AudioFrameSize * VideoHeader.FPS
        return floats.round((drate / 48828.15) * 128) as ubyte
    }

sub Seek_VideoFrame(float FrameNumber) -> float {
      if FrameNumber < 2.0 or FrameNumber > (NumFrames-1) { return 0 }
      uword LeadSize = 32
      uword FrameTotalSize = FrameBufferSize + PalBufferSize
      if VideoHeader.VTYPE==6 or VideoHeader.VTYPE==8 {
         FrameTotalSize += VideoHeader.AudioFrameSize
         ; also include the extra Audio Frame inserted if AudioFrameSize <= 50% of the Audio FIFO
         if VideoHeader.AudioFrameSize < 2049 { LeadSize += VideoHeader.AudioFrameSize }
         }
      return (FrameNumber * FrameTotalSize as float) + LeadSize as float
   }

sub Get_VidFrameNumber(float FilePos) -> float {
  uword HeaderSize = 32
  uword FrameTotalSize = FrameBufferSize + PalBufferSize
   if VideoHeader.VTYPE==6 or VideoHeader.VTYPE==8 {
      FrameTotalSize += VideoHeader.AudioFrameSize
      if VideoHeader.AudioFrameSize < 2049 { HeaderSize += VideoHeader.AudioFrameSize }
    }
   return floats.floor((FilePos - HeaderSize as float)/FrameTotalSize as float) - 1.0
  }

sub GetVideoPosition() {
    my_f_tell(VideoLFN)
    vidFilePosition = (65536.0 * cx16.r1 as float) + cx16.r0 as float
  }

sub my_f_tell(ubyte channel) {
        ; gets the (32 bits) position + file size of the opened read file channel
        ubyte[2] command = ['T',0]
        command[1] = channel       ; f_open uses this secondary address
        cbm.SETNAM(sizeof(command), &command)
        cbm.SETLFS(15, drivenumber, 15)
        void cbm.OPEN()
        void cbm.CHKIN(15)        ; use #15 as input channel
        success=false
        ; valid response starts with "07," followed by hex notations of the position and filesize
        if cbm.CHRIN()=='0' and cbm.CHRIN()=='7' and cbm.CHRIN()==',' {
            cx16.r1 = diskio.read4hex()
            cx16.r0 = diskio.read4hex()        ; position in R1:R0
            void cbm.CHRIN()            ; separator space
            cx16.r3 = diskio.read4hex()
            cx16.r2 = diskio.read4hex()        ; filesize in R3:R2
            success = true
        }
        %asm {{ jmp p8s_Flush15 }}
    }

sub f_seek(ubyte channel, float seekpos) {
        ; gets the (32 bits) position + file size of the opened read file channel
        cx16.r2 = floats.floor(seekpos/65536.0) as uword
        cx16.r1 = (seekpos - (cx16.r2 as float * 65536.0)) as uword
        cx16.r0 = mkword(channel,'P')   ; complete building the P command
        cbm.SETNAM(6, &cx16.r0)
        cbm.SETLFS(15, drivenumber, 15)
        void cbm.OPEN()
              ; use #15 as input channel
        %asm {{
              jmp p8s_Flush15
            }}
       }

sub OpenAudioFile() {
    strings.copy(BASENAME, AUDFILE)
    strings.append(AUDFILE,AEXT)
    if FExists(AUDFILE,strings.length(AUDFILE))
     {
       fopen(SoundLFN, drivenumber,SoundLFN, AUDFILE)
       my_f_tell(SoundLFN)
       AudFileSize = (65536.0 * f_size_high as float) + f_size_low as float
       success = true
       return }
         else {
            ; open failed
            success = false
            return }
    }

sub CheckBookMark() -> bool {
    strings.copy(BASENAME, AUDFILE)
    strings.append(AUDFILE,".MARK")
    if FExists(AUDFILE,strings.length(AUDFILE)) {
       fopen(7,drivenumber,7,AUDFILE)
       %asm {{
               ldx #7
               jsr cbm.CHKIN
               ldx #<p8v_Frame
               ldy #>p8v_Frame
               lda #5
               jsr cx16.MACPTR
            }}
       cbm.CLRCHN()
       cbm.CLOSE(7)
       return true
     }
   return false
  }

sub SetVideoAttributes() {
    UseBookMark = CheckBookMark()
    if VideoHeader.VTYPE==1 or VideoHeader.VTYPE==3 {
       OpenAudioFile()
       VideoHeader.AudioFrameSize=calc_asize(AudFileSize, NumFrames)
       VideoHeader.VERARate = CalculateVERARate()
      }
    XOrg = 0
    YOrg = 0

    ABuff_255 = (VideoHeader.AudioFrameSize / 255 ) as ubyte
    ABuff_Remainder = (VideoHeader.AudioFrameSize - (ABuff_255 as uword * 255)) as ubyte

    if ABuff_Remainder==0 { ABuff_255 -= 1 ABuff_Remainder=255 }  ; adjust for rare case where AFrame is evenly divisible by 255

    CalcFrameSize()
    if VideoHeader.VTYPE==1 or VideoHeader.VTYPE==6 { CalculateSprites() }

    ubyte APreLoops = 1 if (VideoHeader.AudioFrameSize<2048 and  (not UseBookMark)) { APreLoops +=1 }

    cx16.rambank(10)
    Engine_SetAudioCode(mkword(ABuff_255,APreLoops), mkword(ABuff_255, ABuff_Remainder), ABuff_Remainder)

    if VideoHeader.VTYPE==1 or VideoHeader.VTYPE==3 { SetType(0) }
    if VideoHeader.VTYPE==6 or VideoHeader.VTYPE==8 { SetAudioLFN(VideoLFN) SetType(1)}

    if VideoHeader.VTYPE==1 or VideoHeader.VTYPE==6 {
         if VideoHeader.BPP==4 { SetSpriteMovie4bit(mkword(VBuff_Remainder,VBuff_255), NumSprites as ubyte, SpriteAttr_EndAddress, SpriteBlock2_DataAddress) }
         if VideoHeader.BPP==8 { SetSpriteMovie8bit(mkword(VBuff_Remainder,VBuff_255), NumSprites as ubyte, SpriteAttr_EndAddress, SpriteBlock2_DataAddress) }
       }

     if VideoHeader.VTYPE==3 or VideoHeader.VTYPE==8 {
        ubyte WidthVal = 0 if VideoHeader.FrameWidth==640 { WidthVal=1 }
        SetBitMapMovie(mkword(VBuff_Remainder,VBuff_255), WidthVal, PalBufferSize as ubyte)
     }

     if UseBookMark {
       f_seek(VideoLFN, Seek_VideoFrame(Frame))
       if VideoHeader.VTYPE==1 or VideoHeader.VTYPE==3 { f_seek(SoundLFN, ((Frame-2) * VideoHeader.AudioFrameSize as float) )  }
      }
    }

sub ResetAllLFNs() {
    cbm.CLOSE(VideoLFN)
    cbm.CLOSE(SoundLFN)
    cbm.CLOSE(4)
    cbm.CLOSE(6)
    cbm.CLOSE(7)
    }

sub CleanUpMarkFile() {
    strings.copy(BASENAME,AUDFILE)
    strings.append(AUDFILE,".MARK")
    if FExists(AUDFILE,strings.length(AUDFILE)) { scratch(AUDFILE) }
 }



sub parse_header() -> bool {
            if VideoHeader.ID[0]== main.FILEID[0]  and VideoHeader.ID[1]== main.FILEID[1]  and VideoHeader.ID[2]== main.FILEID[2]  and VideoHeader.ID[3]== main.FILEID[3] {
            if VideoHeader.VTYPE==1 or VideoHeader.VTYPE==3 or VideoHeader.VTYPE==6 or VideoHeader.VTYPE==8 {
                NumFrames = (65536.0 * VideoHeader.NumFramesHI as float) + VideoHeader.NumFramesLO as float
                return true }}
          }

sub OpenVideo() {
    fopen(VideoLFN, drivenumber , VideoLFN, VIDFILE)
    %asm {{
            ldx p8v_VideoLFN
            jsr cbm.CHKIN
            ldx #0
            ldy #$05
            lda #32
            clc
            jsr cx16.MACPTR
            jsr cbm.CLRCHN
          }}
          if parse_header() { SetVideoAttributes() }
    }

sub hFullScale(uword HRes) -> ubyte {
  float hScale = floats.round((HRes as float/640.0) * 128.0)
  return hScale as ubyte
 }

sub vFullScale(uword VRes) -> ubyte {
  float vScale = floats.round((VRes as float/480.0) * 128.0)
  return vScale as ubyte
 }

sub SetFullScreen() {
    ubyte vScale = vFullScale(VideoHeader.FrameHeight)
    ubyte hScale = hFullScale(VideoHeader.FrameWidth)
    if VideoHeader.FrameWidth as float <= (VideoHeader.FrameHeight as float/1.5) {
       hScale = vScale - 1
     }
   cx16.VERA_DC_HSCALE = hScale
   cx16.VERA_DC_VSCALE = vScale
  }

sub setPlayerScreen() {
    if VideoHeader.BPP<8 {
       cx16.VERA_ADDR_L = $C0
       cx16.VERA_ADDR_M = $FB
       cx16.VERA_ADDR_H = %00010001
       repeat 64 { cx16.VERA_DATA0 = 0 }
     }
    ubyte smode = $80
    when VideoHeader.VTYPE {
        1, 6 -> if VideoHeader.BPP==8 {smode=3}
        3, 8 -> if VideoHeader.FrameWidth==640 { smode=0 }
       }
     cx16.set_screen_mode(smode)
     if (VideoHeader.VTYPE==3 or VideoHeader.VTYPE==8) and VideoHeader.FrameWidth==640 { showGraphicsLayer() }
     if (VideoHeader.VTYPE==1 or VideoHeader.VTYPE==6) and (VideoHeader.BPP==4 and FrameBufferSize>16384) { hideTextLayer() }
     cx16.rambank(10)
     ClearAllBuffers()
     if VideoHeader.FrameWidth as float <= (VideoHeader.FrameHeight as float/1.5) {
       word VisPixels = floats.round(((vFullScale(VideoHeader.FrameHeight) - 1) as float / 128.0) * 640) as word
       XOrg = (VisPixels - VideoHeader.FrameWidth) / 2
     }

     if VideoHeader.VTYPE==1 or VideoHeader.VTYPE==6 { setup_SpriteGrid(XOrg,YOrg) }

     if VideoHeader.VTYPE==3 or VideoHeader.VTYPE==8 {
         when VideoHeader.BPP {
            1 -> cx16.VERA_L0_CONFIG = %00000100
            2 -> cx16.VERA_L0_CONFIG = %00000101
            4 -> cx16.VERA_L0_CONFIG = %00000110
          }
        }
     SetFullScreen()
    }


sub start() {
    VideoLFN = 4
    SoundLFN = 6

    if Volume==0 or Volume>15 { Volume = 10 }
    txt.cp437()
loadlib:
    cx16.rambank(10)
    cx16.memory_copy(playengine, $A000, &endengine-&playengine)

    void diskio.fastmode(3)
    cx16.set_screen_mode(1)
getfilename:
    ISOUpper(BASENAME)


ProcessFileName:
    VIDFILE = " "
    strings.copy(BASENAME, VIDFILE)
    strings.append(VIDFILE,VEXT)
    txt.print("\x0D")
    txt.print("\x05\x0D ")

    if FExists(VIDFILE,strings.length(VIDFILE)) {
      txt.print(VIDFILE)
      txt.print("\x99 FOUND !\x0D\x0D")
LoadTheVideo:
      txt.clear_screen()
      OpenVideo()


PlayTheVideo:
      InVideo = true
      setPlayerScreen()
      cx16.rambank(10)
      PlayVideo(mkword(VERAcfgvar,VideoHeader.VERARate))
      pal2black()
      sprites.reset(1,127)
      showTextLayer()
      SetDefaultPalette()
      goto skiptoend
      }
      txt.print("\x96 NOT FOUND !\x0D\x05")
      InVideo = false
      Beep() sys.wait(90)
      goto alldone
 skiptoend:
    if ManualExit==$FF {
       DoBookMark = true
       GetVideoPosition()
       Frame  = Get_VidFrameNumber(vidFilePosition)
       strings.copy("@:", AUDFILE)
       strings.append(AUDFILE,BASENAME)
       strings.append(AUDFILE,".MARK,S,W")
       fopen(7,drivenumber,7,AUDFILE)
       %asm {{
               ldx #7
               jsr cbm.CHKOUT
               ldx #<p8v_Frame
               ldy #>p8v_Frame
               lda #5
               jsr cx16.MCIOUT
            }}
       cbm.CLRCHN()
       cbm.CLOSE(7)
    } else {
          CleanUpMarkFile()
          DoBookMark = false
        }
      Volume = AUDIO_CTRL & $0F
      ResetAllLFNs()
      cx16.set_screen_mode(1)
alldone:
   cbm.CLOSE(VideoLFN)
   cbm.CLOSE(SoundLFN)
   pal2black()
   cx16.set_screen_mode(1)
   SetDefaultPalette()
   txt.iso_off()
   txt.uppercase()
   txt.color2(1,6)
   txt.cls()
   ProgramBanner()

skipoutputstuff1:
   txt.row(10)
   txt.column(2)
   txt.color2(10,6)
   if InVideo  txt.print(BASENAME) txt.color(7)
   ResetAllLFNs()
   txt.print(" DONE \x05!\x0D")
   if DoBookMark {
    txt.color(5)
    txt.print("  BOOKMARK AT FRAME\x05:\x99 ") floats.print(Frame)
    }
   txt.color(1)
   cbm.CHROUT(13)
 }
}
